import os


def create_structure(base_path, structure):
    """
    Create a folder structure based on a dictionary.

    Args:
        base_path (str): The base directory where the structure will be created.
        structure (dict): A dictionary defining the folder and file structure.
    """
    for name, content in structure.items():
        path = os.path.join(base_path, name)

        if isinstance(content, dict):
            # If the content is a dictionary, it could be subfolders or files with contents
            os.makedirs(path, exist_ok=True)
            create_structure(path, content)
        elif isinstance(content, list):
            # If the content is a list of files, create empty files
            os.makedirs(base_path, exist_ok=True)
            for file_name in content:
                file_path = os.path.join(base_path, file_name)
                with open(file_path, 'w', encoding="utf-8") as file:
                    file.write("")
        elif isinstance(content, str):
            # If the content is a string, treat it as file content
            os.makedirs(base_path, exist_ok=True)
            with open(path, 'w', encoding="utf-8") as file:
                file.write(content)  # Write the content to the file


if __name__ == "__main__":
    folder_structure = {
        "Folder1": {
            "Subfolder1": {
                "file1.txt": "Lorem ipsum dolor sit amet, consectetur adipiscing elit. \n Morbi a quam lectus. Morbi porttitor odio vitae mollis malesuada.\n In rhoncus orci vitae mi tincidunt auctor. Proin suscipit felis vel augue varius, vel fringilla elit aliquet. Donec at vehicula purus, non dapibus arcu. Phasellus auctor ligula vitae fermentum dapibus. Suspendisse auctor velit dui",

                "file2.txt": "Glownym celem utworzenia Bazy jest upowszechnianie dorobku naukowego pracownikow, doktorantow i studentow PW.  W systemie sa gromadzone i archiwizowane dane dotyczace rożnych aspektow działalności naukowo badawczej pracownikow danej instytucji"
            },
            "Subfolder2": [],
        },
        "Folder2": {
            "file3.txt": "Przykładowe dane w pliku nr 3"
        },
        "Folder3": {
            "Subfolder3": {
                "Subsubfolder1": {
                    "file4.txt": "Humpty Dumpty sat on a wall. \nHumpty Dumpty had a great fall. \nAll the king’s horses and all the king’s men \nCouldn’t put Humpty together again."
                }
            }
        },
        "file5.txt": "Przykładowe dane w pliku nr 5"
    }

    base_directory = "example-source-dir"
    create_structure(base_directory, folder_structure)
    print(f"Folder structure created in '{base_directory}'")
