use pyo3::prelude::*;
use std::fs::File;
use std::io::prelude::*;


#[pyfunction]
fn check_simple_regex(path: String, pattern: String) -> PyResult<bool> {
    let mut file = File::open(path.clone()).expect("Cannot open!");
    let mut buffer = Vec::new();
    file.read_to_end(&mut buffer)?;
    if std::str::from_utf8(&buffer).is_err() {
        return Ok(false);
    }
    file.rewind().unwrap();
    let mut contents = String::new();
    file.read_to_string(&mut contents);
        // .expect("Cannot read file");
    Ok(is_match(&contents, &pattern))
}

fn is_match(string: &str, pattern: &str) -> bool {
    let s_chars: Vec<char> = string.chars().collect();
    let p_chars: Vec<char> = pattern.chars().collect();
    let mut starting_index = 0;
    let mut found = false;
    while (!found && starting_index < s_chars.len()) {
        
        if is_match_helper(&s_chars, &p_chars, starting_index, 0){
            found = true;
        }
        starting_index += 1;
    }
    found
}

fn is_match_helper(
    string_chars: &[char],
    patter_chars: &[char],
    string_idx: usize,
    pattern_idx: usize,
) -> bool {
    // if pattern is exhausted, the specified regex exists in the string
    if pattern_idx == patter_chars.len() {
        return true;
    }
    if string_idx == string_chars.len() {
        return false;
    }
    // Check if the current characters of the string and pattern match
    let current_match = string_idx < string_chars.len()
        && (string_chars[string_idx] == patter_chars[pattern_idx]
            || patter_chars[pattern_idx] == '.');

    // Check if the current pattern character is modified by "*"
    if pattern_idx + 1 < patter_chars.len() && patter_chars[pattern_idx + 1] == '*' {
        // either match to the rest of the pattern or continue matching to character preceding the "*"
        return is_match_helper(string_chars, patter_chars, string_idx, pattern_idx + 2)
            || (current_match
                && is_match_helper(string_chars, patter_chars, string_idx + 1, pattern_idx));
    }
   
    // move to the next characters in both the string and pattern
    current_match && is_match_helper(string_chars, patter_chars, string_idx + 1, pattern_idx + 1)
}

#[pymodule]
fn text_detector(m: &Bound<'_, PyModule>) -> PyResult<()> {
    m.add_function(wrap_pyfunction!(check_simple_regex, m)?)?;
    Ok(())
}

#[cfg(test)]
#[path = "./test/test.rs"]
mod test;
